searchNodes=[{"doc":"","ref":"RustlerElixirFun.html","title":"RustlerElixirFun","type":"module"},{"doc":"This GenServer listens for native code to send sets of functions + parameters to it. It will then execute these functions (by calling apply(fun, parameters) on them), and return the results to the native code. iex&gt; { :ok , pid } = RustlerElixirFun.FunExecutionServer . start_link ( [ ] ) iex&gt; RustlerElixirFun.Internal . apply_elixir_fun ( pid , fn x -&gt; x * 2 end , [ 10 ] ) { :ok , 20 } It is recommended to use a registered name (and start the process in a supervision tree) to make sure that you won't have to handle checking PIDs for aliveness in native code: iex&gt; { :ok , _ } = RustlerElixirFun.FunExecutionServer . start_link ( [ name : :my_fancy_worker_server ] ) iex&gt; RustlerElixirFun.Internal . apply_elixir_fun ( :my_fancy_worker_server , fn x -&gt; x * 2 end , [ 42 ] ) { :ok , 84 } If you might call a function many times in short succession, or want to call many different functions, it might make sense to use RustlerElixirFun.Pool instead.","ref":"RustlerElixirFun.FunExecutionServer.html","title":"RustlerElixirFun.FunExecutionServer","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"RustlerElixirFun.FunExecutionServer.html#child_spec/1","title":"RustlerElixirFun.FunExecutionServer.child_spec/1","type":"function"},{"doc":"","ref":"RustlerElixirFun.FunExecutionServer.html#start_link/1","title":"RustlerElixirFun.FunExecutionServer.start_link/1","type":"function"},{"doc":"","ref":"RustlerElixirFun.Internal.html","title":"RustlerElixirFun.Internal","type":"module"},{"doc":"Calls the Rust NIF directly. This is mainly useful to simplify testing; in your actual projects, you'll want to call the Rust function directly from your Rust code.","ref":"RustlerElixirFun.Internal.html#apply_elixir_fun/3","title":"RustlerElixirFun.Internal.apply_elixir_fun/3","type":"function"},{"doc":"This Supervisor manages a pool of worker processes which will each listen to native code sending sets of functions + parameters. Currently uses the default options: [ name : RustlerElixirFun.Pool , size : 50 , max_overflow : 20 ] Note that when starting, the 'name' will be used as registered name for the RustlerElixirFun.Pool.PoolMaster process. You can use this name directly in the native Rust code. (If you ever need to interact with the poolboy pool itself: its name will be name + .PoolboyPool ). iex&gt; Supervisor . start_link ( RustlerElixirFun.Pool , [ name : MyPool ] ) iex&gt; ( ( 1 .. 100 ) iex&gt; |&gt; Task . async_stream ( fn i -&gt; RustlerElixirFun.Internal . apply_elixir_fun ( MyPool , fn x -&gt; x * 2 end , [ i ] ) end , max_concurrency : 50 ) iex&gt; |&gt; Enum . to_list iex&gt; |&gt; Enum . map ( fn { :ok , { :ok , val } } -&gt; val end ) iex&gt; |&gt; Enum . sum ( ) ) 10100","ref":"RustlerElixirFun.Pool.html","title":"RustlerElixirFun.Pool","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"RustlerElixirFun.Pool.html#child_spec/1","title":"RustlerElixirFun.Pool.child_spec/1","type":"function"},{"doc":"","ref":"RustlerElixirFun.Pool.html#start_link/1","title":"RustlerElixirFun.Pool.start_link/1","type":"function"},{"doc":"Delegates work to the various workers in the pool. This module is used internally when interacting with RustlerElixirFun.Pool .","ref":"RustlerElixirFun.Pool.PoolMaster.html","title":"RustlerElixirFun.Pool.PoolMaster","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"RustlerElixirFun.Pool.PoolMaster.html#child_spec/1","title":"RustlerElixirFun.Pool.PoolMaster.child_spec/1","type":"function"},{"doc":"","ref":"RustlerElixirFun.Pool.PoolMaster.html#start_link/1","title":"RustlerElixirFun.Pool.PoolMaster.start_link/1","type":"function"},{"doc":"Calling an Elixir or Erlang function from native Rust code. This is a work-in-progress.","ref":"readme.html","title":"RustlerElixirFun","type":"extras"},{"doc":"rustler_elixir_fun is split into two parts: an Elixir library, and a Rust library. In a project where you are writing your own NIF, you'll need both. First, add the Rust crate to your Crate.toml : [ dependencies ] rustler_elixir_fun = &quot;0.1.0&quot; Secondly, add the Elixir library to your mix.exs : def deps do [ { :rustler_elixir_fun , &quot;~&gt; 0.1.0&quot; } ] end","ref":"readme.html#installation","title":"RustlerElixirFun - Installation","type":"extras"},{"doc":"Make sure you have started either RustlerElixirFun.FunExecutionServer or RustlerElixirFun.Pool depending on how often you expect calls to be made. Using a pool is slightly slower but does not have a single-process bottleneck. You'll probably want to start one or the other of these under your supervision tree, as a named process (See the Elixir documentation for more details). Now, in your Rust code, you'll want to call rustler_elixir_fun::apply_elixir_fun(env, pid_or_name, fun, parameters) . This will, assuming that pid_or_name refers to the server or pool you've started earlier, run the Elixir function fun by passing it the parameters , returning its result. See the Rust documentation for more details.","ref":"readme.html#basic-usage","title":"RustlerElixirFun - Basic usage","type":"extras"},{"doc":"Elixir documentation can be found at https://hexdocs.pm/rustler_elixir_fun . Rust documentation can be found at https://docs.rs/rustler_elixir_fun/ .","ref":"readme.html#documentation","title":"RustlerElixirFun - Documentation","type":"extras"},{"doc":"In diagram form: In text form: In native code, create a manual 'future' (a mutex wrapping a potentially-empty value, and a condition variable). Wrap this 'future' into a reference which can be passed back to Elixir. Send {fun, params, future_ref} to a particular process &lt;small&gt;(or pool of processes)&lt;/small&gt; which you've been running. Block the native code until the future is filled. &lt;small&gt;(Or a timeout is triggered)&lt;/small&gt; In the elixir GenServer, run the function, &lt;small&gt;(handle errors)&lt;/small&gt; and once the result is obtained, call a second NIF with the result and the future_ref. This NIF will 'fill' the future with the passed result and immediately return. Now the original native code will continue.","ref":"readme.html#how-does-it-work","title":"RustlerElixirFun - How does it work?","type":"extras"},{"doc":"A simple benchmark has been set up to check the overhead of calling a simple function ( fn x -&gt; x * x end ) directly from elixir (using apply(fun, [some_number]) ) vs calling a NIF which will use rustler_elixir_fun to call back into Elixir. The outcome: It is not super fast, but for many situations it will be fast enough. It takes roughly 85 times longer, at 9.5µs vs 0.11µs. This overhead, while not neglegible, might still be considered ‘good enough’ for many projects, as an overhead of 9.5µs will often be overshadowed if there is some actual work (such as communication with other processes, IO, etc.) going on inside the function. Run these benchmarks yourself by (after cloning the repo) running MIX_ENV = bench mix run bench / call_overhead . ex $ MIX_ENV=bench mix run bench/call_overhead.ex Operating System: Linux CPU Information: Intel(R) Core(TM) i7-6700HQ CPU @ 2.60GHz Number of Available Cores: 8 Available memory: 31.18 GB Elixir 1.12.0 Erlang 24.0.1 Benchmark suite executing with the following configuration: warmup: 2 s time: 5 s memory time: 0 ns reduction time: 0 ns parallel: 1 inputs: 100, 100_000, 100_000_000 Estimated total run time: 1.40 min Benchmarking apply(fun, param) with input 100 ... Benchmarking apply(fun, param) with input 100_000 ... Benchmarking apply(fun, param) with input 100_000_000 ... Benchmarking apply_elixir_fun(Pool, fun, param) with input 100 ... Benchmarking apply_elixir_fun(Pool, fun, param) with input 100_000 ... Benchmarking apply_elixir_fun(Pool, fun, param) with input 100_000_000 ... Benchmarking apply_elixir_fun(Server, fun, param) with input 100 ... Benchmarking apply_elixir_fun(Server, fun, param) with input 100_000 ... Benchmarking apply_elixir_fun(Server, fun, param) with input 100_000_000 ... Benchmarking fun.(param) with input 100 ... Benchmarking fun.(param) with input 100_000 ... Benchmarking fun.(param) with input 100_000_000 ... ##### With input 100 ##### Name ips average deviation median 99th % apply(fun, param) 8.65 M 0.116 μs ±144.37% 0.114 μs 0.178 μs fun.(param) 8.64 M 0.116 μs ±147.69% 0.114 μs 0.156 μs apply_elixir_fun(Pool, fun, param) 0.104 M 9.63 μs ±222.87% 8.54 μs 27.16 μs apply_elixir_fun(Server, fun, param) 0.101 M 9.95 μs ±233.02% 8.68 μs 27.38 μs Comparison: apply(fun, param) 8.65 M fun.(param) 8.64 M - 1.00x slower +0.00003 μs apply_elixir_fun(Pool, fun, param) 0.104 M - 83.27x slower +9.51 μs apply_elixir_fun(Server, fun, param) 0.101 M - 86.01x slower +9.83 μs ##### With input 100_000 ##### Name ips average deviation median 99th % apply(fun, param) 8.55 M 0.117 μs ±140.47% 0.115 μs 0.160 μs fun.(param) 8.41 M 0.119 μs ±131.56% 0.115 μs 0.180 μs apply_elixir_fun(Server, fun, param) 0.104 M 9.59 μs ±209.01% 8.53 μs 26.84 μs apply_elixir_fun(Pool, fun, param) 0.0994 M 10.06 μs ±179.97% 8.80 μs 27.30 μs Comparison: apply(fun, param) 8.55 M fun.(param) 8.41 M - 1.02x slower +0.00198 μs apply_elixir_fun(Server, fun, param) 0.104 M - 82.02x slower +9.47 μs apply_elixir_fun(Pool, fun, param) 0.0994 M - 86.05x slower +9.94 μs ##### With input 100_000_000 ##### Name ips average deviation median 99th % fun.(param) 8.42 M 0.119 μs ±114.30% 0.115 μs 0.185 μs apply(fun, param) 8.27 M 0.121 μs ±141.67% 0.118 μs 0.25 μs apply_elixir_fun(Server, fun, param) 0.103 M 9.75 μs ±219.29% 8.53 μs 27.54 μs apply_elixir_fun(Pool, fun, param) 0.0961 M 10.40 μs ±193.83% 8.94 μs 27.95 μs Comparison: fun.(param) 8.42 M apply(fun, param) 8.27 M - 1.02x slower +0.00213 μs apply_elixir_fun(Server, fun, param) 0.103 M - 82.08x slower +9.63 μs apply_elixir_fun(Pool, fun, param) 0.0961 M - 87.54x slower +10.28 μs","ref":"readme.html#how-fast-is-it","title":"RustlerElixirFun - How fast is it?","type":"extras"},{"doc":"I did not come up with this on my own. Very clever people have recognized this approach before. Thank you, Erlang Forums user @robashton and GitHub user @tessi for your explanations and inspiration!","ref":"readme.html#attribution-thanks","title":"RustlerElixirFun - Attribution &amp; Thanks","type":"extras"}]